package flow.stages

import chisel3._
import flow.Params
import chisel3.util.Decoupled
import chisel3.util.DecoupledIO
import chisel3.util.ReadyValidIO

class DecoupledMsgIO[Tin <: Data, Tout <: Data](
    _in: => Option[DecoupledIO[Tin]],
    _out: => Option[DecoupledIO[Tout]]
)(implicit
    p: Params
) extends Bundle {
  implicit class optionWithGetThrow[T](o: Option[T]) {
    def getOrThrow[E <: Exception](e: E): T = o match {
      case Some(x) => x
      case None    => throw e
    }
  }
  lazy val in = _in.getOrThrow(new Exception(s"No input port at $this"))
  lazy val out = _out.getOrThrow(new Exception(s"No output port at $this"))

  def connect[Tout_ <: Data](
      other: DecoupledMsgIO[Tout, Tout_]
  ) = {
    if (p.arch == "single") {
      // out.ready := DontCare
      // out.valid := DontCare
      // other.in.valid := DontCare
      // other.in.ready := DontCare
      other.in.bits := out.bits
    } else if (p.arch == "multi") {
      out :<>= other.in
    } else {
      throw new Exception("Unknown architecture")
    }
    other
  }
}

object DecoupledMsgIO {
  import scala.language.implicitConversions

  /** Wrap input and output messages in ReadValidIO
    */
  def apply[Tin <: Data, Tout <: Data](
      in: Option[Tin] = None,
      out: Option[Tout] = None,
      isIO: Boolean = true
  )(implicit
      p: Params
  ) = {
    val _in = in match {
      case Some(d) =>
        if (isIO) Some(Flipped(Decoupled(d)))
        else Some(Wire(Flipped(Decoupled(d))))
      case None => None
    }

    val _out = out match {
      case Some(d) =>
        if (isIO) Some(Decoupled(d)) else Some(Decoupled(d))
      case None => None
    }
    new DecoupledMsgIO(_in, _out)
  }
}

object utils {
  implicit class dataWrapBySome[T <: Data](d: T) {
    def S = Some(d)
  }
}
