package flow.stages

import chisel3._
import chisel3.util.Decoupled
import chisel3.util.DecoupledIO
import flow.Params
import flow.stages.utils._
import flow.components.newALU
import flow.components.ALUControlInterface
import flow.stages.messages._
import flow.components.RV32InstSubfields._

class EX(implicit val p: Params) extends Module {

  // val msgio = IO(DecoupledMsgIO((new ID2EX).S, (new EX2LS).S))
  val msgio = IO(new Bundle {
    val in = Flipped(DecoupledIO(new ID2EX))
    val out = DecoupledIO(new EX2LS)
  })
  msgio.in.ready := DontCare
  msgio.out.valid := DontCare

  val io = IO(new Bundle {
    val toIF = Output(new EX2IF)
  })

  private val _toIF = io.toIF
  private val _in = msgio.in.bits
  private val _out = msgio.out.bits

  val alu = Module(new newALU)
  alu.control := _in.aluCtrl;

  {
    import flow.components.ALUControlInterface.SrcASelect._
    import flow.components.util.chiselEnumAsInt
    alu.in.a(aSrcARs1) := _in.src1
    alu.in.a(aSrcAPc) := _in.pc
    alu.in.a(aSrcAZero) := 0.U

    import flow.components.ALUControlInterface.SrcBSelect._
    alu.in.b(aSrcBRs2) := _in.src2
    alu.in.b(aSrcBImmI) := _in.inst.immI
    alu.in.b(aSrcBImmJ) := _in.inst.immJ
    alu.in.b(aSrcBImmU) := _in.inst.immU
    alu.in.b(aSrcBImmS) := _in.inst.immS
  }

  _out.exeOut := alu.out.result

  _toIF.jAddr := alu.out.result
  _toIF.brOffset := _in.inst.immB
  _toIF.pc := _in.pc

  import flow.components.newPcControlInterface.SrcSelect._
  val regSrcEq = Wire(Bool());
  regSrcEq := (_in.src1 === _in.src2);
  when(_in.pcCtrl.srcSelect === pBR) {
    val branchUseSlt = _in.inst(14)
    val branchInvertResult = _in.inst(12)
    val _branchResult =
      Mux(branchUseSlt, alu.out.result(0), regSrcEq)
    val branchResult = Mux(branchInvertResult, !_branchResult, _branchResult)
    _toIF.pcCtrl.srcSelect := Mux(branchResult, pBR, pStatic)
  }.otherwise {
    _toIF.pcCtrl.srcSelect := _in.pcCtrl.srcSelect
  }

  _out.ramCtrl := _in.ramCtrl
  _out.pc := _in.pc
  _out.rd := _in.inst.rd
  _out.src1 := _in.src1
  _out.src2 := _in.src2
  _out.regCtrl := _in.regCtrl
}
