package flow.stages.messages

import chisel3._
import flow.Params
import flow.components._

class IF2ID(implicit p: Params) extends Bundle {
  val inst = UInt(p.instWidth)
  val pc = UInt(p.XLEN)
}

class ID2EX(implicit p: Params) extends Bundle {
  // Data
  val pc = UInt(p.XLEN)
  val inst = UInt(p.XLEN)
  val src1 = UInt(p.XLEN)
  val src2 = UInt(p.XLEN)

  // Control
  val aluCtrl = Flipped(ALUControlInterface())
  val ramCtrl = Flipped(new DpiRamControlInterface)
  val pcCtrl = Flipped(newPcControlInterface())
  val regCtrl = Flipped(RegControl())
}

class EX2LS(implicit p: Params) extends Bundle {
  val pc = UInt(p.XLEN)
  val rd = UInt(p.regsAddrWidth)
  val src1 = UInt(p.XLEN)
  val src2 = UInt(p.XLEN)

  val exeOut = UInt(p.XLEN)

  // Control
  val ramCtrl = Flipped(new DpiRamControlInterface)
  val regCtrl = Flipped(RegControl())
}

class LS2WB(implicit p: Params) extends Bundle {
  val pc = UInt(p.XLEN)
  val rd = UInt(p.regsAddrWidth)
  val memOut = UInt(p.XLEN)
  val exeOut = UInt(p.XLEN)

  val regCtrl = Flipped(RegControl())
}

class EX2IF(implicit p: Params) extends Bundle {
  val pc = UInt(p.XLEN)
  val brOffset = UInt(p.XLEN)
  val jAddr = UInt(p.XLEN)

  // Control
  val pcCtrl = Flipped(newPcControlInterface())
}

class IF2Ram(implicit p: Params) extends Bundle {
  val pc = UInt(p.XLEN)
}

class Ram2IF(implicit p: Params) extends Bundle {
  val inst = UInt(p.XLEN)
}

/** Commit result back to registers
  */
class WB2ID(implicit p: Params) extends Bundle {
  val regCtrl = Flipped(RegControl())
  val exeOut = UInt(p.XLEN)
  val memOut = UInt(p.XLEN)
  val rd = UInt(p.regsAddrWidth)
  val npc = UInt(p.XLEN)
}
