package flow.tests

import chisel3._
import chiseltest._
import org.scalatest.freespec.AnyFreeSpec
import chiseltest.simulator.WriteVcdAnnotation

import flow.tests.defaultParams
import flow.stages.utils._
import flow.stages.DecoupledMsgIO
import chisel3.util.Decoupled

class StageConnect extends AnyFreeSpec with ChiselScalatestTester {
  "should compile" in {
    implicit val p: flow.Params = defaultParams().copy(arch = "single")

    class stage1 extends Module {
      val io = DecoupledMsgIO(out = (new Bundle {
        val data = UInt(12.W)
      }).S)
      io.out.valid := true.B
      io.out.bits.data := 1.U
    }

    class stage2 extends Module {
      val io = DecoupledMsgIO(Some(new Bundle {
        val data = UInt(12.W)
      }))
      io.in.ready := true.B
    }

    class stage3 extends Module {
      val wireOut = DecoupledMsgIO(
        out = Some(new Bundle {
          val data = UInt(12.W)
        }),
        isIO = false
      )
      val wireIn = DecoupledMsgIO(
        Some(new Bundle {
          val data = UInt(12.W)
        }),
        isIO = false
      )
      wireOut connect [Nothing] wireIn

      wireOut.out.valid := true.B
      wireOut.out.bits.data := 1.U
      wireIn.in.ready := true.B
    }

    class stage extends Module {
      val s1 = Module(new stage1)
      val s2 = Module(new stage2)
      s1.io connect [Nothing] s2.io
    }

    import circt.stage.ChiselStage
    println(ChiselStage.emitSystemVerilog(new stage1))

    test(new stage) { c =>
      println(c)
    }

    test(new stage3) { c =>
      println(c)
    }

  }
}
